<?php

class ModelExtensionModuleConnectix extends Model
{
    use \ctx\traits\Db;
    use \ctx\traits\LibraryLoader;

    private $events = array(
        'catalog/model/checkout/order/addOrderHistory/before' => array(
            'extension/module/connectix/beforeAddOrderHistory'
        ),
        'catalog/model/checkout/order/addOrderHistory/after'  => array(
            'extension/module/connectix/afterAddOrderHistory'
        )
    );

    /**
     * @return array
     */
    public function fetchTemplates()
    {
        return $this->db->query("SELECT * FROM " . DB_PREFIX . "connectix_templates")->rows;
    }

    /**
     * @param array|null $templates
     * @return array
     */
    public function upsertTemplates($templates)
    {
        if (!empty($templates)) {
            $templateIds = array_map(
                function ($template) {
                    return $template['id'];
                },
                $templates
            );
        } else {
            $templateIds = [];
        }

        $fields = ['media', 'layout', 'name', 'channel', 'text', 'button_label', 'button_url', 'status'];
        $formatData = function ($templateData) use ($fields) {
            $data = [];
            foreach ($fields as $field) {
                $data[$field] = isset($templateData[$field]) ? $this->db->escape($templateData[$field]) : null;
            }

            return $data;
        };

        $isChanged = function ($old, $new) use ($fields) {
            $changed = false;
            foreach ($fields as $field) {
                if (!isset($new[$field])) {
                    continue;
                }

                if ($old[$field] !== (string)$new[$field]) {
                    $changed = true;
                    break;
                }
            }

            return $changed;
        };

        $forDeletion = $forUpdate = [];
        $existingTemplates = $this->fetchTemplates();
        foreach ($existingTemplates as $template) {
            if (!in_array($template['provider_id'], $templateIds)) {
                $forDeletion[] = $template['id'];
            } elseif (false !== $index = array_search($template['provider_id'], array_column($templates, 'id'))) {
                if ($isChanged($template, $templates[$index])) {
                    $forUpdate[$template['id']] = $formatData($templates[$index]);
                }
            }
        }

        $fetchFresh = false;
        // Handle removals
        if (!empty($forDeletion)) {
            $fetchFresh = true;
            $this->qb->query('DELETE FROM ' . DB_PREFIX . 'connectix_templates WHERE id IN (' . $this->db->escape(implode(',', $forDeletion)) . ')');
        }

        // Handle updates
        foreach ($forUpdate as $id => $data) {
            $fetchFresh = true;
            $updateSet = [];
            foreach ($data as $key => $value) {
                $updateSet[] = "`$key` = '$value'";
            }
            $this->db->query("UPDATE " . DB_PREFIX . "connectix_templates SET " . implode(', ', $updateSet) . " WHERE id = '" . (int)$id . "'");
        }

        // Handle insertions
        foreach ($templates as $template) {
            if (array_search($template['id'], array_column($existingTemplates, 'provider_id')) === false) {
                $fetchFresh = true;
                $data = $formatData($template);
                $data['provider_id'] = $template['id'];
                $insertSet = [];
                foreach ($data as $key => $value) {
                    $insertSet['columns'][] = "`$key`";
                    $insertSet['values'][] = "'$value'";
                }
                $this->db->query('INSERT INTO ' . DB_PREFIX . 'connectix_templates (' . implode(',', $insertSet['columns']) . ') VALUES (' . implode(',', $insertSet['values']) . ')');
            }
        }

        // Fetch fresh data if it's necessary
        return $fetchFresh ? $this->fetchTemplates() : $existingTemplates;
    }

    /**
     * @param array $postData
     * @return array|mixed|null
     */
    public function saveRule(array $postData)
    {
        $data = [];
        foreach (['enabled', 'fallback', 'type', 'delivery_type', 'order_status', 'days_not_picked_up'] as $field) {
            $value = isset($postData[$field]) ? $this->db->escape($postData[$field]) : null;
            switch ($value) {
                case 'null':
                    $value = null;
                    break;
                case 'false':
                case 'true':
                    $value = $value === 'true' ? 1 : 0;
                    break;
            }
            $data[$field] = $value;
        }

        if (isset($postData['id'])) {
            $id = (int)$postData['id'];
            $this->db->query('DELETE FROM ' . DB_PREFIX . 'connectix_rules_templates WHERE rule_id = ' . (int)$this->db->escape($id));
            $sqlData = $this->prepareDataForUpdate($data);
            $this->db->query('UPDATE ' . DB_PREFIX . 'connectix_rules SET ' . $sqlData . ' WHERE id = ' . (int)$this->db->escape($id));
        } else {
            $sqlData = $this->prepareDataForPersist($data);
            $this->db->query('INSERT INTO ' . DB_PREFIX . 'connectix_rules (' . $sqlData['columns'] . ') VALUES (' . $sqlData['values'] . ')');
            $id = $this->db->getLastId();
        }

        foreach ($postData['templates'] as $i => $templateId) {
            $sqlData = $this->prepareDataForPersist([
                'template_id' => $templateId,
                'rule_id'     => $id,
                'position'    => $i,
            ]);
            $this->db->query('INSERT INTO ' . DB_PREFIX . 'connectix_rules_templates (' . $sqlData['columns'] . ') VALUES (' . $sqlData['values'] . ')');
        }

        $record = $this->fetchRules(null, $id);
        if (!empty($record)) {
            $record = $record[0];
        }

        return $record;
    }

    /**
     * @param $id
     * @return string
     */
    public function deleteRule($id)
    {
        $this->db->query('DELETE FROM ' . DB_PREFIX . 'connectix_rules WHERE id = ' . (int)$this->db->escape($id));
        return 'OK';
    }

    public function createTables()
    {
        $this->db->query("CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "connectix_message_inbounds (id INT AUTO_INCREMENT NOT NULL, message_id INT DEFAULT NULL, text LONGTEXT NOT NULL, sent_at DATETIME DEFAULT NULL, INDEX IDX_CEBA6A1C537A1329 (message_id), PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8mb4 COLLATE `utf8mb4_general_ci` ENGINE = InnoDB;");
        $this->db->query("CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "connectix_messages (id INT AUTO_INCREMENT NOT NULL, order_id INT NOT NULL, text VARCHAR(1200) NOT NULL, status VARCHAR(255) NOT NULL, channel VARCHAR(10) DEFAULT NULL, price NUMERIC(10, 4) NOT NULL, provider_id VARCHAR(64) DEFAULT NULL, fallback_id VARCHAR(64) DEFAULT NULL, created_at DATETIME DEFAULT NULL, type VARCHAR(255) DEFAULT NULL, delivery_type VARCHAR(6) NOT NULL, PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8mb4 COLLATE `utf8mb4_general_ci` ENGINE = InnoDB;");
        $this->db->query("CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "connectix_rules (id INT AUTO_INCREMENT NOT NULL, fallback TINYINT(1) NOT NULL, type VARCHAR(25) NOT NULL, order_status INT DEFAULT NULL, delivery_type VARCHAR(6) DEFAULT NULL, enabled TINYINT(1) NOT NULL, days_not_picked_up INT DEFAULT NULL, PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8mb4 COLLATE `utf8mb4_general_ci` ENGINE = InnoDB;");
        $this->db->query("CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "connectix_rules_templates (id INT AUTO_INCREMENT NOT NULL, template_id INT NOT NULL, rule_id INT NOT NULL, position INT NOT NULL, INDEX IDX_37FB6BF5DA0FB8 (template_id), INDEX IDX_37FB6BF744E0351 (rule_id), PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8mb4 COLLATE `utf8mb4_general_ci` ENGINE = InnoDB;");
        $this->db->query("CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "connectix_templates (id INT AUTO_INCREMENT NOT NULL, media VARCHAR(255) DEFAULT NULL, layout VARCHAR(25) NOT NULL, name VARCHAR(255) NOT NULL, channel VARCHAR(10) NOT NULL, text LONGTEXT NOT NULL, button_label VARCHAR(255) DEFAULT NULL, button_url VARCHAR(255) DEFAULT NULL, status VARCHAR(255) NOT NULL, provider_id CHAR(36) NOT NULL COMMENT '(DC2Type:uuid)', UNIQUE INDEX UNIQ_FAB48C01A53A8AA (provider_id), PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8mb4 COLLATE `utf8mb4_general_ci` ENGINE = InnoDB;");
        $this->db->query("CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "connectix_tracking (id INT AUTO_INCREMENT NOT NULL, order_id INT NOT NULL, tracking_number VARCHAR(32) NOT NULL, PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8mb4 COLLATE `utf8mb4_general_ci` ENGINE = InnoDB;");
        $this->db->query("CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "connectix_order_token (id INT AUTO_INCREMENT NOT NULL, order_id INT NOT NULL, token VARCHAR(32) NOT NULL, status VARCHAR(10) DEFAULT NULL, created_at DATETIME NOT NULL, updated_at DATETIME NOT NULL, used TINYINT(1) NOT NULL, PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8mb4 COLLATE `utf8mb4_general_ci` ENGINE = InnoDB;");
    }

    public function createIndexes()
    {
        $this->db->query("ALTER TABLE " . DB_PREFIX . "connectix_message_inbounds ADD CONSTRAINT FK_CEBA6A1C537A1329 FOREIGN KEY (message_id) REFERENCES " . DB_PREFIX . "connectix_messages (id) ON DELETE CASCADE;");
        $this->db->query("ALTER TABLE " . DB_PREFIX . "connectix_rules_templates ADD CONSTRAINT FK_37FB6BF5DA0FB8 FOREIGN KEY (template_id) REFERENCES " . DB_PREFIX . "connectix_templates (id) ON DELETE CASCADE;");
        $this->db->query("ALTER TABLE " . DB_PREFIX . "connectix_rules_templates ADD CONSTRAINT FK_37FB6BF744E0351 FOREIGN KEY (rule_id) REFERENCES " . DB_PREFIX . "connectix_rules (id) ON DELETE CASCADE;");
    }

    public function dropTables()
    {
        $this->db->query("ALTER TABLE " . DB_PREFIX . "connectix_message_inbounds DROP FOREIGN KEY FK_CEBA6A1C537A1329;");
        $this->db->query("ALTER TABLE " . DB_PREFIX . "connectix_rules_templates DROP FOREIGN KEY FK_37FB6BF744E0351;");
        $this->db->query("ALTER TABLE " . DB_PREFIX . "connectix_rules_templates DROP FOREIGN KEY FK_37FB6BF5DA0FB8;");
        $this->db->query("DROP TABLE " . DB_PREFIX . "connectix_message_inbounds;");
        $this->db->query("DROP TABLE " . DB_PREFIX . "connectix_messages;");
        $this->db->query("DROP TABLE " . DB_PREFIX . "connectix_rules;");
        $this->db->query("DROP TABLE " . DB_PREFIX . "connectix_rules_templates;");
        $this->db->query("DROP TABLE " . DB_PREFIX . "connectix_templates;");
        $this->db->query("DROP TABLE " . DB_PREFIX . "connectix_tracking;");
        $this->db->query("DROP TABLE " . DB_PREFIX . "connectix_order_token;");
    }

    public function deleteEvents()
    {
        $this->load->model('setting/event');

        $this->model_setting_event->deleteEventByCode('module_connectix');
    }

    public function createEvents()
    {
        $this->load->model('setting/event');

        foreach ($this->events as $trigger => $actions) {
            foreach ($actions as $action) {
                $this->model_setting_event->addEvent('module_connectix', $trigger, $action, 1, 0);
            }
        }
    }

    public function addTrackingNumber($order_id, $tracking_number)
    {
        $tracking = $this->db->query("SELECT id FROM `" . DB_PREFIX . "connectix_tracking` WHERE order_id = '" . (int)$order_id . "'");
        if ($tracking->num_rows > 0) {
            $this->db->query("UPDATE " . DB_PREFIX . "connectix_tracking SET tracking_number = '" . $this->db->escape($tracking_number) . "' WHERE id = '" . (int)$tracking->row['id'] . "'");
        } else {
            $this->db->query("INSERT INTO " . DB_PREFIX . "connectix_tracking SET order_id = '" . (int)$order_id . "', tracking_number = '" . $this->db->escape($tracking_number) . "'");
        }

        if (!$this->registry->has('connectix')) {
            $this->loadLibrary($this->store_id);
        }

        $library = $this->registry->get('connectix');
        $library->actionObjectOrderCarrierUpdate($tracking_number);
    }

    public function getTrackingNumber($order_id)
    {
        $tracking = $this->db->query("SELECT tracking_number FROM `" . DB_PREFIX . "connectix_tracking` WHERE order_id = '" . (int)$order_id . "'");

        return $tracking->num_rows > 0 ? $tracking->row['tracking_number'] : '';
    }
}
